package javax.management.loading;

import java.lang.reflect.Array;
import java.io.ObjectStreamClass;
import java.io.StreamCorruptedException;
import java.io.IOException;
import java.io.InputStream;
import java.io.ObjectInputStream;

class MLetObjectInputStream extends ObjectInputStream
{
    private MLet loader;
    
    public MLetObjectInputStream(InputStream in, MLet loader) throws IOException, StreamCorruptedException {
        super(in);
        if (loader == null) {
            throw new IllegalArgumentException("Illegal null argument to MLetObjectInputStream");
        }
        this.loader = loader;
        return;
    }
    
    private Class<?> primitiveType(char c) {
        switch (c) {
            case 'B': {
                return Byte.TYPE;
            }
            case 'C': {
                return Character.TYPE;
            }
            case 'D': {
                return Double.TYPE;
            }
            case 'F': {
                return Float.TYPE;
            }
            case 'I': {
                return Integer.TYPE;
            }
            case 'J': {
                return Long.TYPE;
            }
            case 'S': {
                return Short.TYPE;
            }
            case 'Z': {
                return Boolean.TYPE;
            }
            default: {
                return null;
            }
        }
    }
    
    @Override
    protected Class<?> resolveClass(ObjectStreamClass objectStreamClass) throws IOException, ClassNotFoundException {
        String name = objectStreamClass.getName();
        if (name.startsWith("[")) {
            int index;
            for (index = 1; name.charAt(index) == '['; ++index) {}
            Class<Object> componentType;
            if (name.charAt(index) == 'L') {
                componentType = this.loader.loadClass(name.substring(index + 1, name.length() - 1));
            }
            else {
                if (name.length() != index + 1) {
                    throw new ClassNotFoundException(name);
                }
                componentType = this.primitiveType(name.charAt(index));
            }
            int[] dimensions = new int[index];
            for (int i = 0; i < index; ++i) {
                dimensions[i] = 0;
            }
            return Array.newInstance(componentType, dimensions).getClass();
        }
        else {
            return this.loader.loadClass(name);
        }
    }
    
    public ClassLoader getClassLoader() {
        return this.loader;
    }
}
